@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from __future__ import unicode_literals
 
 import sys
@@ -7,23 +6,17 @@ from datetime import date, datetime
 from decimal import Decimal, ROUND_UP, ROUND_DOWN
 from unittest import TestCase
 
-from wtforms import validators, widgets
+from wtforms import validators, widgets, meta
 from wtforms.fields import *
-from wtforms.fields import Label, Field, SelectFieldBase
+from wtforms.fields import Label, Field, SelectFieldBase, html5
 from wtforms.form import Form
 from wtforms.compat import text_type
+from wtforms.utils import unset_value
+from tests.common import DummyPostData
 
 PYTHON_VERSION = sys.version_info
 
 
-class DummyPostData(dict):
-    def getlist(self, key):
-        v = self[key]
-        if not isinstance(v, (list, tuple)):
-            v = [v]
-        return v
-
-
 class AttrDict(object):
     def __init__(self, *args, **kw):
         self.__dict__.update(*args, **kw)
@@ -79,7 +72,6 @@ class LabelTest(TestCase):
         self.assertEqual(label(**{'for': 'bar'}), """<label for="bar">Caption</label>""")
 
 
-
 class FlagsTest(TestCase):
     def setUp(self):
         t = TextField(validators=[validators.Required()]).bind(Form(), 'a')
@@ -111,15 +103,25 @@ class FlagsTest(TestCase):
         self.assertEqual(self.flags._foo, 42)
 
 
+class UnsetValueTest(TestCase):
+    def test(self):
+        self.assertEqual(str(unset_value), '<unset value>')
+        self.assertEqual(repr(unset_value), '<unset value>')
+        self.assertEqual(bool(unset_value), False)
+        assert not unset_value
+        self.assertEqual(unset_value.__nonzero__(), False)
+        self.assertEqual(unset_value.__bool__(), False)
+
+
 class FiltersTest(TestCase):
     class F(Form):
         a = TextField(default=' hello', filters=[lambda x: x.strip()])
-        b = TextField(default='42', filters=[lambda x: int(x)])
+        b = TextField(default='42', filters=[int, lambda x: -x])
 
     def test_working(self):
         form = self.F()
         self.assertEqual(form.a.data, 'hello')
-        self.assertEqual(form.b.data, 42)
+        self.assertEqual(form.b.data, -42)
         assert form.validate()
 
     def test_failure(self):
@@ -159,6 +161,19 @@ class FieldTest(TestCase):
         Field.process_formdata(self.field, [42])
         self.assertEqual(self.field.data, 42)
 
+    def test_meta_attribute(self):
+        # Can we pass in meta via _form?
+        form = self.F()
+        assert form.a.meta is form.meta
+
+        # Can we pass in meta via _meta?
+        form_meta = meta.DefaultMeta()
+        field = TextField(_name='Foo', _form=None, _meta=form_meta)
+        assert field.meta is form_meta
+
+        # Do we fail if both _meta and _form are None?
+        self.assertRaises(TypeError, TextField, _name='foo', _form=None)
+
 
 class PrePostTestField(TextField):
     def pre_validate(self, form):
@@ -202,7 +217,7 @@ class PrePostValidationTest(TestCase):
 
 class SelectFieldTest(TestCase):
     class F(Form):
-        a = SelectField(choices=[('a', 'hello'), ('btest','bye')], default='a')
+        a = SelectField(choices=[('a', 'hello'), ('btest', 'bye')], default='a')
         b = SelectField(choices=[(1, 'Item 1'), (2, 'Item 2')], coerce=int, option_widget=widgets.TextInput())
 
     def test_defaults(self):
@@ -230,8 +245,10 @@ class SelectFieldTest(TestCase):
         form = self.F()
         first_option = list(form.a)[0]
         self.assertTrue(isinstance(first_option, form.a._Option))
-        self.assertEqual(list(text_type(x) for x in form.a), ['<option selected value="a">hello</option>',
-                                                            '<option value="btest">bye</option>'])
+        self.assertEqual(
+            list(text_type(x) for x in form.a),
+            ['<option selected value="a">hello</option>', '<option value="btest">bye</option>']
+        )
         self.assertTrue(isinstance(first_option.widget, widgets.Option))
         self.assertTrue(isinstance(list(form.b)[0].widget, widgets.TextInput))
         self.assertEqual(first_option(disabled=True), '<option disabled selected value="a">hello</option>')
@@ -247,7 +264,7 @@ class SelectFieldTest(TestCase):
 
 class SelectMultipleFieldTest(TestCase):
     class F(Form):
-        a = SelectMultipleField(choices=[('a', 'hello'), ('b','bye'), ('c', 'something')], default=('a', ))
+        a = SelectMultipleField(choices=[('a', 'hello'), ('b', 'bye'), ('c', 'something')], default=('a', ))
         b = SelectMultipleField(coerce=int, choices=[(1, 'A'), (2, 'B'), (3, 'C')], default=("1", "3"))
 
     def test_defaults(self):
@@ -277,12 +294,12 @@ class SelectMultipleFieldTest(TestCase):
         self.assertEqual(form.b.data, None)
         form = self.F(DummyPostData(b=['fake']))
         assert not form.validate()
-        self.assertEqual(form.b.data, [1,3])
+        self.assertEqual(form.b.data, [1, 3])
 
 
 class RadioFieldTest(TestCase):
     class F(Form):
-        a = RadioField(choices=[('a', 'hello'), ('b','bye')], default='a')
+        a = RadioField(choices=[('a', 'hello'), ('b', 'bye')], default='a')
         b = RadioField(choices=[(1, 'Item 1'), (2, 'Item 2')], coerce=int)
 
     def test(self):
@@ -290,16 +307,39 @@ class RadioFieldTest(TestCase):
         self.assertEqual(form.a.data, 'a')
         self.assertEqual(form.b.data, None)
         self.assertEqual(form.validate(), False)
-        self.assertEqual(form.a(), """<ul id="a"><li><input checked id="a-0" name="a" type="radio" value="a"> <label for="a-0">hello</label></li><li><input id="a-1" name="a" type="radio" value="b"> <label for="a-1">bye</label></li></ul>""")
-        self.assertEqual(form.b(), """<ul id="b"><li><input id="b-0" name="b" type="radio" value="1"> <label for="b-0">Item 1</label></li><li><input id="b-1" name="b" type="radio" value="2"> <label for="b-1">Item 2</label></li></ul>""")
-        self.assertEqual([text_type(x) for x in form.a], ['<input checked id="a-0" name="a" type="radio" value="a">', '<input id="a-1" name="a" type="radio" value="b">'])
+        self.assertEqual(
+            form.a(),
+            (
+                """<ul id="a">"""
+                """<li><input checked id="a-0" name="a" type="radio" value="a"> <label for="a-0">hello</label></li>"""
+                """<li><input id="a-1" name="a" type="radio" value="b"> <label for="a-1">bye</label></li></ul>"""
+            )
+        )
+        self.assertEqual(
+            form.b(),
+            (
+                """<ul id="b">"""
+                """<li><input id="b-0" name="b" type="radio" value="1"> <label for="b-0">Item 1</label></li>"""
+                """<li><input id="b-1" name="b" type="radio" value="2"> <label for="b-1">Item 2</label></li></ul>"""
+            )
+        )
+        self.assertEqual(
+            [text_type(x) for x in form.a],
+            ['<input checked id="a-0" name="a" type="radio" value="a">', '<input id="a-1" name="a" type="radio" value="b">']
+        )
 
     def test_text_coercion(self):
         # Regression test for text coercsion scenarios where the value is a boolean.
         coerce_func = lambda x: False if x == 'False' else bool(x)
         F = make_form(a=RadioField(choices=[(True, 'yes'), (False, 'no')], coerce=coerce_func))
         form = F()
-        self.assertEqual(form.a(), '<ul id="a"><li><input id="a-0" name="a" type="radio" value="True"> <label for="a-0">yes</label></li><li><input checked id="a-1" name="a" type="radio" value="False"> <label for="a-1">no</label></li></ul>')
+        self.assertEqual(
+            form.a(),
+            '''<ul id="a">'''
+            '''<li><input id="a-0" name="a" type="radio" value="True"> <label for="a-0">yes</label></li>'''
+            '''<li><input checked id="a-1" name="a" type="radio" value="False"> <label for="a-1">no</label></li></ul>'''
+        )
+
 
 class TextFieldTest(TestCase):
     class F(Form):
@@ -315,6 +355,7 @@ class TextFieldTest(TestCase):
         form = self.F(DummyPostData(b=['hello']))
         self.assertEqual(form.a.data, '')
 
+
 class HiddenFieldTest(TestCase):
     class F(Form):
         a = HiddenField(default="LE DEFAULT")
@@ -520,14 +561,11 @@ class DateTimeFieldTest(TestCase):
         self.assertFalse(form.validate())
         self.assertEqual(form.a.errors[0], 'Not a valid datetime value')
 
-        form = self.F(a=d,b=d)
+        form = self.F(a=d, b=d)
         self.assertTrue(form.validate())
         self.assertEqual(form.a._value(), '2008-05-05 04:30:00')
 
     def test_microseconds(self):
-        if PYTHON_VERSION < (2, 6):
-            return # Microsecond formatting support was only added in 2.6
-
         d = datetime(2011, 5, 7, 3, 23, 14, 424200)
         F = make_form(a=DateTimeField(format='%Y-%m-%d %H:%M:%S.%f'))
         form = F(DummyPostData(a=['2011-05-07 03:23:14.4242']))
@@ -545,14 +583,14 @@ class SubmitFieldTest(TestCase):
 class FormFieldTest(TestCase):
     def setUp(self):
         F = make_form(
-            a = TextField(validators=[validators.required()]),
-            b = TextField(),
+            a=TextField(validators=[validators.required()]),
+            b=TextField(),
         )
-        self.F1 = make_form('F1', a = FormField(F))
-        self.F2 = make_form('F2', a = FormField(F, separator='::'))
+        self.F1 = make_form('F1', a=FormField(F))
+        self.F2 = make_form('F2', a=FormField(F, separator='::'))
 
     def test_formdata(self):
-        form = self.F1(DummyPostData({'a-a':['moo']}))
+        form = self.F1(DummyPostData({'a-a': ['moo']}))
         self.assertEqual(form.a.form.a.name, 'a-a')
         self.assertEqual(form.a['a'].data, 'moo')
         self.assertEqual(form.a['b'].data, '')
@@ -574,7 +612,13 @@ class FormFieldTest(TestCase):
         self.assertEqual(obj_inner.b, None)
 
     def test_widget(self):
-        self.assertEqual(self.F1().a(), '''<table id="a"><tr><th><label for="a-a">A</label></th><td><input id="a-a" name="a-a" type="text" value=""></td></tr><tr><th><label for="a-b">B</label></th><td><input id="a-b" name="a-b" type="text" value=""></td></tr></table>''')
+        self.assertEqual(
+            self.F1().a(),
+            '''<table id="a">'''
+            '''<tr><th><label for="a-a">A</label></th><td><input id="a-a" name="a-a" type="text" value=""></td></tr>'''
+            '''<tr><th><label for="a-b">B</label></th><td><input id="a-b" name="a-b" type="text" value=""></td></tr>'''
+            '''</table>'''
+        )
 
     def test_separator(self):
         form = self.F2(DummyPostData({'a-a': 'fake', 'a::a': 'real'}))
@@ -586,12 +630,14 @@ class FormFieldTest(TestCase):
         class A(Form):
             a = FormField(self.F1, validators=[validators.required()])
         self.assertRaises(TypeError, A)
+
         class B(Form):
             a = FormField(self.F1, filters=[lambda x: x])
         self.assertRaises(TypeError, B)
 
         class C(Form):
             a = FormField(self.F1)
+
             def validate_a(form, field):
                 pass
         form = C()
@@ -605,7 +651,6 @@ class FormFieldTest(TestCase):
         form.populate_obj(obj2)
 
 
-
 class FieldListTest(TestCase):
     t = TextField(validators=[validators.Required()])
 
@@ -638,7 +683,7 @@ class FieldListTest(TestCase):
     def test_enclosed_subform(self):
         make_inner = lambda: AttrDict(a=None)
         F = make_form(
-            a = FieldList(FormField(make_form('FChild', a=self.t), default=make_inner))
+            a=FieldList(FormField(make_form('FChild', a=self.t), default=make_inner))
         )
         data = [{'a': 'hello'}]
         form = F(a=data)
@@ -667,7 +712,7 @@ class FieldListTest(TestCase):
         self.assertRaises(TypeError, form.populate_obj, obj2)
 
     def test_entry_management(self):
-        F = make_form(a = FieldList(self.t))
+        F = make_form(a=FieldList(self.t))
         a = F(a=['hello', 'bye']).a
         self.assertEqual(a.pop_entry().name, 'a-1')
         self.assertEqual(a.data, ['hello'])
@@ -679,11 +724,11 @@ class FieldListTest(TestCase):
         self.assertRaises(IndexError, a.pop_entry)
 
     def test_min_max_entries(self):
-        F = make_form(a = FieldList(self.t, min_entries=1, max_entries=3))
+        F = make_form(a=FieldList(self.t, min_entries=1, max_entries=3))
         a = F().a
         self.assertEqual(len(a), 1)
         self.assertEqual(a[0].data, None)
-        big_input = ['foo',  'flaf', 'bar', 'baz']
+        big_input = ['foo', 'flaf', 'bar', 'baz']
         self.assertRaises(AssertionError, F, a=big_input)
         pdata = DummyPostData(('a-%d' % i, v) for i, v in enumerate(big_input))
         a = F(pdata).a
@@ -697,7 +742,7 @@ class FieldListTest(TestCase):
             elif len(field.data) > 2:
                 raise ValueError('too many')
 
-        F = make_form(a = FieldList(self.t, validators=[validator]))
+        F = make_form(a=FieldList(self.t, validators=[validator]))
 
         # Case 1: length checking validators work as expected.
         fdata = DummyPostData({'a-0': ['hello'], 'a-1': ['bye'], 'a-2': ['test3']})
@@ -720,6 +765,27 @@ class FieldListTest(TestCase):
         my_filter = lambda x: x
         self.assertRaises(TypeError, FieldList, self.t, filters=[my_filter], _form=Form(), _name='foo')
 
+    def test_process_prefilled(self):
+        data = ['foo', 'hi', 'rawr']
+
+        class A(object):
+            def __init__(self, a):
+                self.a = a
+        obj = A(data)
+        F = make_form(a=FieldList(self.t))
+        # fill form
+        form = F(obj=obj)
+        self.assertEqual(len(form.a.entries), 3)
+        # pretend to submit form unchanged
+        pdata = DummyPostData({
+            'a-0': ['foo'],
+            'a-1': ['hi'],
+            'a-2': ['rawr']})
+        form.process(formdata=pdata)
+        # check if data still the same
+        self.assertEqual(len(form.a.entries), 3)
+        self.assertEqual(form.a.data, data)
+
 
 class MyCustomField(TextField):
     def process_data(self, data):
@@ -744,6 +810,61 @@ class CustomFieldQuirksTest(TestCase):
         f = self.F()
         self.assertRaises(NotImplementedError, f.b.iter_choices)
 
-if __name__ == '__main__':
-    from unittest import main
-    main()
+
+class HTML5FieldsTest(TestCase):
+    class F(Form):
+        search = html5.SearchField()
+        telephone = html5.TelField()
+        url = html5.URLField()
+        email = html5.EmailField()
+        datetime = html5.DateTimeField()
+        date = html5.DateField()
+        dt_local = html5.DateTimeLocalField()
+        integer = html5.IntegerField()
+        decimal = html5.DecimalField()
+        int_range = html5.IntegerRangeField()
+        decimal_range = html5.DecimalRangeField()
+
+    def _build_value(self, key, form_input, expected_html, data=unset_value):
+        if data is unset_value:
+            data = form_input
+        if expected_html.startswith('type='):
+            expected_html = '<input id="%s" name="%s" %s value="%s">' % (key, key, expected_html, form_input)
+        return {
+            'key': key,
+            'form_input': form_input,
+            'expected_html': expected_html,
+            'data': data
+        }
+
+    def test_simple(self):
+        b = self._build_value
+        VALUES = (
+            b('search', 'search', 'type="search"'),
+            b('telephone', '123456789', 'type="tel"'),
+            b('url', 'http://wtforms.simplecodes.com/', 'type="url"'),
+            b('email', 'foo@bar.com', 'type="email"'),
+            b('datetime', '2013-09-05 00:23:42', 'type="datetime"', datetime(2013, 9, 5, 0, 23, 42)),
+            b('date', '2013-09-05', 'type="date"', date(2013, 9, 5)),
+            b('dt_local', '2013-09-05 00:23:42', 'type="datetime-local"', datetime(2013, 9, 5, 0, 23, 42)),
+            b('integer', '42', '<input id="integer" name="integer" step="1" type="number" value="42">', 42),
+            b('decimal', '43.5', '<input id="decimal" name="decimal" step="any" type="number" value="43.5">', Decimal('43.5')),
+            b('int_range', '4', '<input id="int_range" name="int_range" step="1" type="range" value="4">', 4),
+            b('decimal_range', '58', '<input id="decimal_range" name="decimal_range" step="any" type="range" value="58">', 58),
+        )
+        formdata = DummyPostData()
+        kw = {}
+        for item in VALUES:
+            formdata[item['key']] = item['form_input']
+            kw[item['key']] = item['data']
+
+        form = self.F(formdata)
+        for item in VALUES:
+            field = form[item['key']]
+            render_value = field()
+            if render_value != item['expected_html']:
+                tmpl = 'Field {key} render mismatch: {render_value!r} != {expected_html!r}'
+                raise AssertionError(tmpl.format(render_value=render_value, **item))
+            if field.data != item['data']:
+                tmpl = 'Field {key} data mismatch: {field.data!r} != {data!r}'
+                raise AssertionError(tmpl.format(field=field, **item))
